;------------------------------------------------------------------------------
;-    ATMEL Microcontroller Software Support  -   ROUSSET -
;------------------------------------------------------------------------------
; The software is delivered "AS IS" without warranty or condition of any
; kind, either express, implied or statutory. This includes without
; limitation any warranty or condition with respect to merchantability or
; fitness for any particular purpose, or against the infringements of
; intellectual property rights of others.
;-----------------------------------------------------------------------------
;- File source          : in_db01.s
;- Object               : Table for AT91DB01 initialization
;- Translator           : ARM Software Development Toolkit V2.11a
;-
;- Exported Resources   : InitTableEBI - InitTableStack
;- Imported Resources   : None
;-
;- 1.0 15/09/98 JLV     : Creation
;- 2.0 21/10/98 JCZ     : Clean up.
;- 2.1 16/03/99 JCZ     : SDT V2.5
;-                      : Reserve stack space on top of memory for ICE Debug.
;------------------------------------------------------------------------------

            AREA        Init, CODE, READONLY

; -------------------------- List of Included Files ---------------------------

            INCLUDE     Include/ebi.inc

; -------------------------- List of Macro definitions ------------------------

            MACRO
            STACK_SIZE  $debug, $fiq, $irq, $svc, $abort, $undef

            DCD     $debug*4
            DCD     $fiq*4
            DCD     $irq*4
            DCD     $svc*4
            DCD     $abort*4
            DCD     $undef*4

            MEND

; ------------------------ List of constants and types ------------------------
; None

; ------------------------ List of Imported resources -------------------------
; None

; ------------------------ List of Internal resources -------------------------
; None

; ------------------------ List of Exported resources -------------------------

    IF  :DEF:AT91_DEBUG_NONE

;------------------------------------------------------------------------------
;- Function             : InitTableEBI
;- Treatments           : Table for EBI initialization
;- Input Parameters     : None
;- Output Parameters    : None
;- Registers lost       : None
;- Called Functions     : None
;- Called Macros        : None
;------------------------------------------------------------------------------

                EXPORT      InitTableEBI
; 32,768 MHz master clock assumed
InitTableEBI
            DCD         0x0100222A  ; 0x01000000, 1MB, 1 hold,  8 bits, 6 WS
            DCD         0x02002229  ; 0x02000000, 1MB, 1 hold, 16 bits, 5 WS
            DCD         0x20000000  ; unused
            DCD         0x30000000  ; unused
            DCD         0x40000000  ; unused
            DCD         0x50000000  ; unused
            DCD         0x60000000  ; unused
            DCD         0x70000000  ; unused
            DCD         0x00000001  ; REMAP command
            DCD         0x00000006  ; 7 memory regions, standard read
            DCD         EBI_BASE    ; EBI Base address

    ENDIF

;------------------------------------------------------------------------------
;- Function             : InitTableStack
;- Treatments           : Table for stack initialization ( stack sizes)
;- Input Parameters     : None
;- Output Parameters    : None
;- Registers lost       : None
;- Called Functions     : None
;- Called Macros        : None
;------------------------------------------------------------------------------

                EXPORT      InitTableStack

InitTableStack
    IF  :DEF:AT91_DEBUG_NONE
            STACK_SIZE  0, 3, 3*8, 256, 3, 3
    ENDIF
    IF  :DEF:AT91_DEBUG_ANGEL
            STACK_SIZE  0, 0, 0, 0, 0, 0
    ENDIF
    IF  :DEF:AT91_DEBUG_ICE
            STACK_SIZE  1024, 3, 3*8, 0, 3, 3
    ENDIF

            END
